"""Functionality to install an iPython kernel."""

import os

from pydantic import BaseModel
from rich.progress import Progress

from ou_container_builder.state import State
from ou_container_builder.util import docker_copy_cmd, docker_run_cmd

name = "ipykernel"
STARTUP_KERNEL_SCRIPT = """#!/bin/bash

export PATH=$PATH:$HOME/.local/bin
ipython kernel install --name "Python-3" --user
"""


class Options(BaseModel):
    """Options for the ipykernel pack."""

    pass


def init(state: State) -> None:
    """Initialise packs.ipykernel."""
    state.update({"environment": [{"name": "PYDEVD_DISABLE_FILE_VALIDATION", "value": "1"}]})
    state.update({"packages": {"pip": {"user": ["ipykernel"]}}})


def generate(state: State, progress: Progress) -> None:  # noqa: ARG001
    """Uninstall the default kernel and activate the startup script."""
    os.makedirs(os.path.join("ou-builder-build", "ipykernel"), exist_ok=True)
    with open(
        os.path.join("ou-builder-build", "ipykernel", "110-activating-the-python-kernel"),
        "w",
    ) as out_f:
        out_f.write(STARTUP_KERNEL_SCRIPT)
    state.update(
        {
            "output_blocks": {
                "deploy": [
                    {
                        "block": docker_run_cmd(
                            [
                                "jupyter kernelspec remove -y python3",
                                "chown -f -R $USER:users $HOME/.ipython || true",
                                "chown -f -R $USER:users $HOME/.local || true",
                            ]
                        ),
                        "weight": 191,
                    },
                    {
                        "block": docker_copy_cmd(
                            os.path.join(
                                "ou-builder-build",
                                "ipykernel",
                                "110-activating-the-python-kernel",
                            ),
                            "/usr/share/ou/startup.d/110-activating-the-python-kernel",
                        ),
                        "weight": 192,
                    },
                ]
            }
        }
    )



"""Functionality to install an IRkernel."""

import os

from pydantic import BaseModel
from rich.progress import Progress

from ou_container_builder.state import State
from ou_container_builder.util import docker_copy_cmd

name = "irkernel"
STARTUP_KERNEL_SCRIPT = """#!/bin/bash

PATH=$PATH:/var/lib/ou/python/system/bin/
R -e "IRkernel::installspec()"
"""


class Options(BaseModel):
    """Options for the irkernel pack."""

    pass


def init(state: State) -> None:
    """Initialise packs.irkernel."""
    state.update({"packages": {"apt": {"deploy": ["r-base", "r-cran-irkernel"]}}})


def generate(state: State, progress: Progress) -> None:  # noqa: ARG001
    """Generate the kernel installation script."""
    os.makedirs(os.path.join("ou-builder-build", "irkernel"), exist_ok=True)
    with open(os.path.join("ou-builder-build", "irkernel", "110-activating-the-R-kernel"), "w") as out_f:
        out_f.write(STARTUP_KERNEL_SCRIPT)
    state.update(
        {
            "output_blocks": {
                "deploy": [
                    {
                        "block": docker_copy_cmd(
                            os.path.join("ou-builder-build", "irkernel", "110-activating-the-R-kernel"),
                            "/usr/share/ou/startup.d/110-activating-the-R-kernel",
                        ),
                        "weight": 191,
                    }
                ]
            }
        }
    )



"""Functionality to install JupyterLab."""

from typing import Literal

from pydantic import BaseModel
from rich.progress import Progress

from ou_container_builder.state import State
from ou_container_builder.util import docker_run_cmd

name = "jupyterlab"


class Options(BaseModel):
    """Options for the JupyterLab pack."""

    version: Literal[4] = 4
    announcements: bool = False


def init(state: State) -> None:
    """Initialise packs.jupyterlab."""
    if state["packs"]["jupyterlab"]["version"] == 4:  # noqa: PLR2004
        state.update({"packages": {"pip": {"system": ["jupyterlab>=4.0.2,<5"]}}})
    if not state["packs"]["jupyterlab"]["announcements"]:
        state.update(
            {
                "output_blocks": {
                    "deploy": [
                        {
                            "block": docker_run_cmd(
                                [
                                    '/var/lib/ou/python/system/bin/jupyter labextension disable "@jupyterlab/apputils-extension:announcements"',  # noqa: E501
                                    "chown -f -R $USER:users $HOME/.jupyter || true",
                                ]
                            ),
                            "weight": 1001,
                        }
                    ]
                }
            }
        )


def generate(state: State, progress: Progress) -> None:
    """Unused."""
    pass
